@@ -29,7 +29,7 @@
  * MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
  *
  * IDENTIFICATION
- *	$Header: /cvsroot/pgsql-server/src/pl/plpython/plpython.c,v 1.26.2.1 2002/11/22 16:25:53 tgl Exp $
+ *	$Header: /cvsroot/pgsql-server/src/pl/plpython/plpython.c,v 1.26.2.4 2003/02/13 23:06:19 tgl Exp $
  *
  *********************************************************************
  */
@@ -127,7 +127,8 @@ typedef struct PLyTypeInfo
  */
 typedef struct PLyProcedure
 {
-	char	   *proname;
+	char	   *proname;		/* SQL name of procedure */
+	char	   *pyname;			/* Python name of procedure */
 	TransactionId fn_xmin;
 	CommandId	fn_cmin;
 	PLyTypeInfo result;			/* also used to store info for trigger
@@ -629,7 +630,7 @@ PLy_modify_tuple(PLyProcedure * proc, PyObject * pltd, TriggerData *tdata,
 			modvalues[j] = FunctionCall3(&proc->result.out.r.atts[atti].typfunc,
 										 CStringGetDatum(src),
 				 ObjectIdGetDatum(proc->result.out.r.atts[atti].typelem),
-							Int32GetDatum(tupdesc->attrs[j]->atttypmod));
+							Int32GetDatum(tupdesc->attrs[atti]->atttypmod));
 			modnulls[j] = ' ';
 
 			Py_DECREF(plstr);
@@ -1050,7 +1051,7 @@ static PLyProcedure *
 PLy_procedure_create(FunctionCallInfo fcinfo, bool is_trigger,
 					 HeapTuple procTup, char *key)
 {
-	char		procName[256];
+	char		procName[NAMEDATALEN+256];
 
 	DECLARE_EXC();
 	Form_pg_proc procStruct;
@@ -1073,8 +1074,10 @@ PLy_procedure_create(FunctionCallInfo fcinfo, bool is_trigger,
 		elog(FATAL, "plpython: Procedure name would overrun buffer");
 
 	proc = PLy_malloc(sizeof(PLyProcedure));
-	proc->proname = PLy_malloc(strlen(procName) + 1);
-	strcpy(proc->proname, procName);
+	proc->proname = PLy_malloc(strlen(NameStr(procStruct->proname)) + 1);
+	strcpy(proc->proname, NameStr(procStruct->proname));
+	proc->pyname = PLy_malloc(strlen(procName) + 1);
+	strcpy(proc->pyname, procName);
 	proc->fn_xmin = HeapTupleHeaderGetXmin(procTup->t_data);
 	proc->fn_cmin = HeapTupleHeaderGetCmin(procTup->t_data);
 	PLy_typeinfo_init(&proc->result);
@@ -1235,21 +1238,21 @@ PLy_procedure_compile(PLyProcedure * proc, const char *src)
 	/*
 	 * insert the function code into the interpreter
 	 */
-	msrc = PLy_procedure_munge_source(proc->proname, src);
+	msrc = PLy_procedure_munge_source(proc->pyname, src);
 	crv = PyObject_CallMethod(proc->interp, "r_exec", "s", msrc);
 	free(msrc);
 
 	if ((crv != NULL) && (!PyErr_Occurred()))
 	{
 		int			clen;
-		char		call[256];
+		char		call[NAMEDATALEN+256];
 
 		Py_DECREF(crv);
 
 		/*
 		 * compile a call to the function
 		 */
-		clen = snprintf(call, sizeof(call), "%s()", proc->proname);
+		clen = snprintf(call, sizeof(call), "%s()", proc->pyname);
 		if ((clen < 0) || (clen >= sizeof(call)))
 			elog(ERROR, "plpython: string would overflow buffer.");
 		proc->code = Py_CompileString(call, "<string>", Py_eval_input);
@@ -1321,6 +1324,8 @@ PLy_procedure_delete(PLyProcedure * proc)
 	Py_XDECREF(proc->me);
 	if (proc->proname)
 		PLy_free(proc->proname);
+	if (proc->pyname)
+		PLy_free(proc->pyname);
 	for (i = 0; i < proc->nargs; i++)
 		if (proc->args[i].is_rel == 1)
 		{
@@ -2176,6 +2181,7 @@ PLy_spi_execute_plan(PyObject * ob, PyObject * list, int limit)
 	int			i,
 				rv;
 	PLyPlanObject *plan;
+	char *nulls;
 
 	enter();
 
@@ -2237,6 +2243,8 @@ PLy_spi_execute_plan(PyObject * ob, PyObject * list, int limit)
 
 	if (nargs)
 	{
+		nulls = palloc((nargs + 1) * sizeof(char));
+
 		for (i = 0; i < nargs; i++)
 		{
 			PyObject   *elem,
@@ -2244,23 +2252,39 @@ PLy_spi_execute_plan(PyObject * ob, PyObject * list, int limit)
 			char	   *sv;
 
 			elem = PySequence_GetItem(list, i);
-			so = PyObject_Str(elem);
-			sv = PyString_AsString(so);
+			if (elem != Py_None)
+			{
+					so = PyObject_Str(elem);
+					sv = PyString_AsString(so);
 
-			/*
-			 * FIXME -- if this can elog, we have leak
-			 */
-			plan->values[i] = FunctionCall3(&(plan->args[i].out.d.typfunc),
-											CStringGetDatum(sv),
-						   ObjectIdGetDatum(plan->args[i].out.d.typelem),
-											Int32GetDatum(-1));
+					/*
+					 * FIXME -- if this can elog, we have leak
+					 */
+					plan->values[i] = FunctionCall3(&(plan->args[i].out.d.typfunc),
+													CStringGetDatum(sv),
+													ObjectIdGetDatum(plan->args[i].out.d.typelem),
+													Int32GetDatum(-1));
+					
+					Py_DECREF(so);
+					Py_DECREF(elem);
 
-			Py_DECREF(so);
-			Py_DECREF(elem);
+					nulls[i] = ' ';
+			}
+			else
+			{
+				Py_DECREF(elem);
+				plan->values[i] = (Datum) 0;
+				nulls[i] = 'n';
+			}
 		}
+		nulls[i] = '\0';
+	}
+	else
+	{
+		nulls = NULL;
 	}
 
-	rv = SPI_execp(plan->plan, plan->values, NULL, limit);
+	rv = SPI_execp(plan->plan, plan->values, nulls, limit);
 	RESTORE_EXC();
 
 	for (i = 0; i < nargs; i++)
@@ -2748,9 +2772,12 @@ PLy_output(volatile int level, PyObject * self, PyObject * args)
 }
 
 
-/* Get the last procedure name called by the backend ( the innermost,
+/*
+ * Get the last procedure name called by the backend ( the innermost,
  * If a plpython procedure call calls the backend and the backend calls
  * another plpython procedure )
+ *
+ * NB: this returns SQL name, not the internal Python procedure name
  */
 
 char *
